require "level"
require "keybindings"
require "sprites"
require "status"

function love.load()
    horizontalSpeed = 250
    verticalSpeed = 250
    
    local modes = love.graphics.getModes()
    table.sort(modes, function(a, b) return a.width*a.height < b.width*b.height end)   -- sort from smallest to largest
    -- local largestRatio =  / 
    -- love.graphics.setMode(modes[#modes].width,modes[#modes].height,true,true)
    love.graphics.setMode(800,600,true,true)
end

function love.draw()
    love.graphics.push()
    love.graphics.translate(math.floor(love.graphics.getWidth()/2-hero.width/2),math.floor(love.graphics.getHeight()/2-hero.height/2))
    love.graphics.translate(math.floor(-math.max(math.min(level.maxX - love.graphics.getWidth ()/2 - hero.width /2, hero.x),    level.minX+love.graphics.getWidth ()/2 - hero.width /2)), 
                            math.floor(-math.max(math.min(level.maxY - love.graphics.getHeight()/2 - hero.height/2, hero.y),    level.minY+love.graphics.getHeight()/2 - hero.height/2)))
------------------------------------------------------------------------------------------------------------------------------------------------------
    local currentLevel = {}
    
    for i=1,#level.objects do
        currentLevel[i] = level.objects[i]
    end
    
    -- sort AIs in currentlevel
    table.sort(level.ais, function(a,b) return a.y < b.y end)
    local curAIIndex = 1
    local maxAIIndex = #level.ais
    for i = 1,#currentLevel+maxAIIndex do
        if level.ais[curAIIndex].y+sprites.monster:getHeight() < currentLevel[i].lowerY then
            table.insert(currentLevel, i, level.ais[curAIIndex])
            curAIIndex = curAIIndex + 1
            if curAIIndex > maxAIIndex then
                break
            end
        end
    end
    
    -- sort hero in currentlevel
    for i = 1,#currentLevel+1 do
        if hero.y+sprites.heroSprite:getHeight() < currentLevel[i].lowerY then
            table.insert(currentLevel, i, hero)
            break
        end
    end
    
    -- draw level
    love.graphics.setColor(255,255,255)
    for i =1,#currentLevel do
        if currentLevel[i] == hero then
            if hero.invincible then
                love.graphics.setColor(255,0,0)
            else
                love.graphics.setColor(255,255,255)
            end
            drawHero()
        else
            love.graphics.setColor(255,255,255)
            love.graphics.draw(currentLevel[i].sprite, math.floor(currentLevel[i].x),math.floor(currentLevel[i].y))
        end
    end
    
    -- draw health bars of ais
    for i=1,#level.ais do
        local currentAI = level.ais[i]
        local ratio = currentAI.health/currentAI.maxhealth
        love.graphics.setColor(0,255,0)
        love.graphics.rectangle("fill", currentAI.x, currentAI.y-10, currentAI.sprite:getWidth()*ratio ,7)
        love.graphics.setColor(255,0,0)
        love.graphics.rectangle("fill", currentAI.x + currentAI.sprite:getWidth()*ratio, currentAI.y-10, currentAI.sprite:getWidth() - currentAI.sprite:getWidth()*ratio ,7)
    end
    
    
------------------------------------------------------------------------------------------------------------------------
    love.graphics.pop()
    drawBuffer()
    for i = 1,hero.health do
        love.graphics.draw(sprites.heart, 10+(i-1)*(sprites.heart:getWidth()+10),10)
    end
    if hero.health <= 0 then
        love.graphics.draw(sprites.gameover, love.graphics.getWidth()/2-sprites.gameover:getWidth()/2, love.graphics.getHeight()/2-sprites.gameover:getHeight()/2)
    end
end

function drawHero()
    if hero.moving then
        sprites.heroAnimation:draw(hero.x, hero.y)
    else
        love.graphics.drawq(sprites.heroSprite, love.graphics.newQuad(0,0,50,100, sprites.heroSprite:getWidth(), sprites.heroSprite:getHeight()), hero.x, hero.y) 
    end 
    
    -- draw sword
    if hero.attack then
        love.graphics.push()
        
        if hero.facingDirection == "right" then
            love.graphics.translate(hero.x+45, hero.y+sprites.heroSprite:getHeight()/2+10)
        elseif hero.facingDirection == "left" then
            love.graphics.translate(hero.x,  hero.y+sprites.heroSprite:getHeight()/2+10)
        end
        
        
            love.graphics.translate(sprites.sword:getWidth()/2,sprites.sword:getHeight()/2)
        if hero.facingDirection == "right" then
            love.graphics.rotate(math.pi/2)
        elseif hero.facingDirection == "left" then
            love.graphics.rotate(-math.pi/2)
        end
            love.graphics.translate(-sprites.sword:getWidth()/2,-sprites.sword:getHeight()/2)
        
        
        love.graphics.draw(sprites.sword,0,0)
        love.graphics.pop()
    end
end

function checkKeys(dt)
    hero.moving = false
    for key,_ in pairs(keyMap) do
        if keyMap[key] and keyBindings[key] then
            keyBindings[key](dt)
        end
    end
end

function love.update(dt)
    checkKeys(dt)
    updateAI(dt)
    sprites.heroAnimation:update(dt)
    
    if love.timer.getMicroTime() - hero.timeSinceBeingInvincible > hero.timeBeingInvincible then
        hero.invincible = false
    end
    
end

function love.keypressed(key)
    keyMap[key] = true
    if key == " " then
        hero.attack = true
        local hitDir
        if hero.facingDirection == "right" then hitDir = 1 elseif hero.facingDirection == "left" then hitDir = -1 end
        for i=#level.ais,1,-1 do
            local curAI = level.ais[i]
            if length((curAI.x+curAI.sprite:getWidth()/2)  - (hero.x+sprites.heroAnimation.fw/2), 
                      (curAI.y+curAI.sprite:getHeight()/2) - (hero.y+sprites.heroAnimation.fh/2)) < hero.attackRange
            and dot(hitDir,0, normalize((curAI.x+curAI.sprite:getWidth()/2)  - (hero.x+sprites.heroAnimation.fw/2), 
                                        (curAI.y+curAI.sprite:getHeight()/2) - (hero.y+sprites.heroAnimation.fh/2)) ) > 0
            then
                curAI.health = curAI.health - 1
                if curAI.health < 0 then
                    table.insert(level.objects,
                                  {sprite = sprites.deadmonster,
                                   x = curAI.x,
                                   y = curAI.y,
                                   isDeadMonster = true,
                                   lowerY = curAI.y+sprites.deadmonster:getHeight()})
                    table.remove(level.ais,i)
                    table.sort(level.objects,function(a,b) return a.lowerY<b.lowerY end)
                    
                end
                break
            end
        end
        for i=#level.objects,1,-1 do
            local curObject = level.objects[i]
            if length((curObject.x+curObject.sprite:getWidth()/2)  - (hero.x+sprites.heroAnimation.fw/2), 
                      (curObject.y+curObject.sprite:getHeight()/2) - (hero.y+sprites.heroAnimation.fh/2)) < hero.attackRange
            and dot(hitDir,0, normalize((curObject.x+curObject.sprite:getWidth()/2)  - (hero.x+sprites.heroAnimation.fw/2), 
                      (curObject.y+curObject.sprite:getHeight()/2) - (hero.y+sprites.heroAnimation.fh/2))) > 0
            and not curObject.isDeadMonster
            then
                table.remove(level.objects,i)
                love.audio.play(sounds.sword[math.random(#sounds.sword)])
                -- printStatus("Destroyed a " .. sprites.filenames[curObject.spriteIndex] .. "!")
                local potion = math.random(100)
                if potion < 10 and curObject.spriteIndex==1 then
                    printStatus("You have found potion!")
                    hero.health = hero.health+1
                end
                break -- only destroy one object per hit
            end
        end
    elseif key == "escape" then
        love.event.quit()
    end
end

function love.keyreleased(key)
    keyMap[key] = false
    if key == " " then
        hero.attack = false
    end
end

function dot(v1x,v1y, v2x, v2y)
    return v1x*v2x + v1y*v2y
end

function lengthSqr(x,y)
    return math.pow(x,2) + math.pow(y,2)
end

function length(x,y)
    return math.sqrt(lengthSqr(x,y))
end

function normalize(x,y)
    local l = length(x,y)
    return x/l, y/l
end

function updateAI(dt)
    for i = 1,#level.ais do
        local vecAItoHero = {}
        vecAItoHero.x = hero.x+sprites.heroAnimation.fw/2 - (level.ais[i].x+sprites.monster:getWidth()/2)
        vecAItoHero.y = hero.y+sprites.heroSprite:getHeight() - (level.ais[i].y + sprites.monster:getHeight())
        vecAItoHero.xNorm, vecAItoHero.yNorm = normalize(vecAItoHero.x, vecAItoHero.y)
        
        local heroHasBeenAttackedX = false
        local heroHasBeenAttackedY = false
        
        if math.abs(vecAItoHero.x) > 10 then
            level.ais[i].x = level.ais[i].x + vecAItoHero.xNorm*level.ais[i].speed*dt
        else
            heroHasBeenAttackedX = true
        end
        
        if math.abs(vecAItoHero.y) > 10 then
            level.ais[i].y = level.ais[i].y + vecAItoHero.yNorm*level.ais[i].speed*dt
        else
            heroHasBeenAttackedY = true
        end
        
        if heroHasBeenAttackedX and heroHasBeenAttackedY and not hero.invincible then
            printStatus("You have been attacked!")
            hero.health = hero.health - 1
            hero.invincible = true
            hero.timeSinceBeingInvincible = love.timer.getMicroTime()
            
            love.audio.play(sounds.monsterroar)
            
            hero.x = hero.x + vecAItoHero.xNorm * 30
            hero.y = hero.y + vecAItoHero.yNorm * 30
        end
    end
end